use crate::service::error::Error;
use axum::response::Html;
use minijinja::{path_loader, Environment, Value};
use minijinja_autoreload::AutoReloader;
use pulldown_cmark::Event;
use std::sync::Arc;

#[derive(Clone)]
pub struct Templates {
    pub reloader: Arc<minijinja_autoreload::AutoReloader>,
}

impl Templates {
    pub fn initialize() -> Result<Self, String> {
        let reloader = AutoReloader::new(move |notifier| {
            let mut environment = Environment::new();
            let template_path = "templates";
            environment.set_loader(path_loader(&template_path));
            environment.add_filter("markdown", markdown);
            notifier.watch_path(template_path, true);
            Ok(environment)
        });
        Ok(Self {
            reloader: Arc::new(reloader),
        })
    }

    pub fn render_template<D: serde::Serialize>(
        &self,
        key: &str,
        data: D,
    ) -> Result<String, Error> {
        let env = self.reloader.acquire_env()?;
        let template = env.get_template(key)?;
        let rendered = template.render(&data)?;
        Ok(rendered)
    }

    pub fn render_html<D: serde::Serialize>(
        &self,
        key: &str,
        data: D,
    ) -> Result<Html<String>, Error> {
        let result = self.render_template(key, data)?;
        Ok(Html(result))
    }
}

fn markdown(value: String) -> Value {
    let mut options = pulldown_cmark::Options::empty();
    options.insert(pulldown_cmark::Options::ENABLE_STRIKETHROUGH);
    options.insert(pulldown_cmark::Options::ENABLE_TABLES);
    options.insert(pulldown_cmark::Options::ENABLE_TASKLISTS);
    options.insert(pulldown_cmark::Options::ENABLE_SMART_PUNCTUATION);
    let parser = pulldown_cmark::Parser::new_ext(&value, options);
    let parser: Box<dyn Iterator<Item = Event>> = Box::new(parser.into_iter());
    let mut html = String::new();
    pulldown_cmark::html::push_html(&mut html, parser);
    Value::from_safe_string(html)
}
